# sb::Content::Entry
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Content::Entry;
use vars qw( $VERSION );
$VERSION = '0.03';
# 0.03 [2007/08/12] renamed from '_main' to '_00main'
# 0.02 [2007/08/07] changed _category because "cat" value has been changed
# 0.01 [2007/08/01] changed _body_text to set description/sequel correctly
# 0.00 [2007/07/31] divided from sb::Content
# --------------------------------------
# // defines / static variables
sub PREV_ARROW (){ '&lt;&lt;' };
sub NEXT_ARROW (){ '&gt;&gt;' };
# --------------------------------------
# // default callbacks
sub _init
{
	return (
		'_00main'   => \&_main,
		'date_time' => \&_date_time,
		'authors'   => \&_authors,
		'attach'    => \&_attachment,
		'category'  => \&_category,
		'body_text' => \&_body_text,
		'discovery' => \&_trackback_auto_discovery,
		'sequel'    => \&_sequel,
		'others'    => \&_others,
	);
}
# --------------------------------------
# // define callbacks
sub _main
{
	my $cms  = shift;
	my %var  = @_;
	undef;
}
sub _date_time
{
	my $cms   = shift;
	my $entry = shift;
	my %var   = @_;
	my $date = sb::Time->format(
		'time'=>$entry->date,
		'form'=>$var{'conf'}->value('conf_entry_date'),
		'zone'=>$entry->tz,
		'lang'=>$var{'conf'}->value('conf_time_lang')
	);
	$cms->tag('entry_date'=>$date);
	my $time = sb::Time->format(
		'time'=>$entry->date,
		'form'=>$var{'conf'}->value('conf_entry_time'),
		'zone'=>$entry->tz,
		'lang'=>$var{'conf'}->value('conf_time_lang')
	);
	my $permalink = sb::Content::Entry::_permalink($entry,$var{'cat'},'',$var{'mode'});
	$cms->tag('entry_time'=>'<a href="' . $permalink . '">' . $time . '</a>');
	$cms->tag('entry_disp_time'=>$time);
}
sub _authors
{
	my $cms   = shift;
	my $entry = shift;
	my %var   = @_;
	$cms->tag('user_name'=>$entry->authlink($var{'user'}));
	$cms->tag('user_disp_name'=>$entry->authname($var{'user'}));
	$cms->tag('user_id'=>$entry->auth);
}
sub _attachment
{ # number of comments, trackbacks
	my $cms   = shift;
	my $entry = shift;
	my %var   = @_;
	if ($entry->acm > 0)
	{
		my $permalink = sb::Content::Entry::_permalink($entry,$var{'cat'},'com',$var{'mode'});
		my $string = $var{'lang'}->string('comments');
		$cms->tag('comment_num'=>'<a href="' . $permalink . '">' . $string . '(' . $entry->com . ')</a>');
		$cms->tag('comment_count'=>$entry->com);
	}
	else
	{
		$cms->tag('comment_num'=>'-');
		$cms->tag('comment_count'=>'');
	}
	if ($entry->atb > 0)
	{
		my $permalink = sb::Content::Entry::_permalink($entry,$var{'cat'},'tb',$var{'mode'});
		my $string = $var{'lang'}->string('trackbacks');
		$cms->tag('trackback_num'=>'<a href="' . $permalink . '">' . $string . '(' . $entry->tb . ')</a>');
		$cms->tag('trackback_count'=>$entry->tb);
	}
	else
	{
		$cms->tag('trackback_num'=>'-');
		$cms->tag('trackback_count'=>'');
	}
}
sub _category
{
	my $cms   = shift;
	my $entry = shift;
	my %var   = @_;
	my $category = ($entry->cat == -1) ? '' : $entry->cat;
	if ( $category ne '' and defined($var{'cat'}->{$category}) )
	{
		$cms->tag('category_name'=>$var{'cat'}->{$category}->fullname_with_link($var{'cat'}));
		$cms->tag('category_id'=>$category);
		$cms->tag('category_disp_name'=>$var{'cat'}->{$category}->fullname($var{'cat'}));
	}
	else
	{
		$cms->tag('category_name'=>'-');
		$cms->tag('category_id'=>'');
		$cms->tag('category_disp_name'=>'-');
	}
	if ( $entry->add ne '' )
	{ # related categories
		my (@text,@with_link);
		foreach ( split(',',$entry->add) )
		{
			next if ($_ eq '');
			next if ( !defined($var{'cat'}->{$_}) );
			push(@with_link,$var{'cat'}->{$_}->fullname_with_link($var{'cat'}));
			push(@text,$var{'cat'}->{$_}->fullname($var{'cat'}));
		}
		$cms->tag('related_category'=>join(', ',@with_link));
		$cms->tag('related_category_disp'=>join(', ',@text));
	}
	else
	{
		$cms->tag('related_category'=>'-');
		$cms->tag('related_category_disp'=>'-');
	}
}
sub _body_text
{ # body, sequel and excerpt
	my $cms   = shift;
	my $entry = shift;
	my %var   = @_;
	my ($body,$more) = ('','');
	if ($entry->body ne '')
	{ # body
		$body = $entry->formatted_body;
		$body .= '<a id="sequel"></a>' if ($entry->more ne '' and $var{'mode'} eq 'ent');
	}
	if ($entry->more ne '')
	{ # sequel
		my $permalink = sb::Content::Entry::_permalink($entry,$var{'cat'},'more',$var{'mode'});
		$more = ($var{'mode'} eq 'ent') 
			? $entry->formatted_more 
			: '<a href="' . $permalink . '">' . $var{'lang'}->string('read more ...') . '</a>';
	}
	$cms->tag('entry_description'=>$body);
	$cms->tag('entry_sequel'=>$more);
	$cms->tag('entry_excerpt'=>$entry->sum);
}
sub _trackback_auto_discovery
{
	my $cms   = shift;
	my $entry = shift;
	my %var   = @_;
	return if ($entry->atb == 0); # trackback is not acceptable, so we ignore this.
	my $permalink = sb::Content::Entry::_permalink($entry,$var{'cat'},'',$var{'mode'});
	my $subject   = $entry->subj;
	my $author    = ( defined($var{'user'}->{$entry->auth}) ) ? $var{'user'}->{$entry->auth}->real : '';
	my $summary   = $entry->sum;
	my $pingurl   = $entry->pingurl;
	my $creatdate = sb::Time->format(
		'time'=>$entry->date,
		'form'=>'%Year%-%Mon%-%Day%T%Hour%:%Min%:%Sec%',
		'zone'=>$var{'conf'}->value('conf_timezone'),
	);
	$summary =~ s/\-\-/\&#45;\&#45;/g if ($summary =~ /\-\-/);
	my $auto_discovery = <<"_TRACKBACK_AUTO_DISCOVERY_";
<!--
<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"
         xmlns:dc=\"http://purl.org/dc/elements/1.1/\"
         xmlns:trackback=\"http://madskills.com/public/xml/rss/module/trackback/\">
<rdf:Description
   rdf:about=\"$permalink\"
   dc:identifier=\"$permalink\"
   dc:title=\"$subject\"
   dc:description=\"$summary\"
   dc:creator=\"$author\"
   dc:date=\"$creatdate\"
   trackback:ping=\"$pingurl\" />
</rdf:RDF>
-->
_TRACKBACK_AUTO_DISCOVERY_
	$cms->tag('trackback_auto_discovery'=>$auto_discovery);
}
sub _sequel
{
	my $cms   = shift;
	my $entry = shift;
	my %var   = @_;
	return if (!$cms->existed('sequel'));
	if ($var{'mode'} eq 'ent' or $var{'mode'} eq 'mob')
	{
		foreach my $label ('next','prev')
		{
			my $entry = $var{'entryinfo'}{'neighbor'}{$label};
			if ($entry)
			{
				my $url  = sb::Content::Entry::_permalink($entry,$var{'cat'},'',$var{'mode'});
				my $subj = $entry->subj;
				my $text = ($label eq 'next') 
				         ? $subj . ' ' . NEXT_ARROW
				         : PREV_ARROW . ' ' . $subj;
				$cms->tag($label . '_permalink'=>$url);
				$cms->tag($label . '_title'=>$subj);
				$cms->tag($label . '_entry'=>'<a href="' . $url . '">' . $text . '</a>');
			}
		}
		$cms->block('sequel'=>1);
	}
	else
	{
		$cms->deleteBlock('sequel');
	}
}
sub _others
{
	my $cms   = shift;
	my $entry = shift;
	my %var   = @_;
	$cms->tag('entry_id'=>$entry->id);
	$cms->tag('permalink'=>sb::Content::Entry::_permalink($entry,$var{'cat'},'',$var{'mode'}));
	$cms->tag('entry_permalink'=>sb::Content::Entry::_permalink($entry,$var{'cat'},'',$var{'mode'}));
	$cms->tag('entry_title'=>$entry->subj);
	$cms->tag('entry_keyword'=>$entry->key);
	$cms->tag('site_parts'=>$var{'conf'}->value('conf_srv_base') . $var{'conf'}->value('dir_style'));
	$cms->tag('sb_entry_marking' => ($var{'conf'}->value('basic_marking') and $var{'mode'} ne 'ent')
		? '<a id="' . $var{'conf'}->value('basic_preid') . $entry->id . '"></a>'
		: '');
}
# --------------------------------------
# // private functions - utilities
sub _permalink
{
	my ($entry,$cat,$mode,$check) = @_;
	my $type = ($check eq 'mob') ? 'Mobile' : sb::Config->get->value('conf_entry_archive');
	$entry->permalink('cat'=>$cat,'type'=>$type,'mode'=>$mode);
}
1;
__END__
