# sb::Content::Feed
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Content::Feed;
use vars qw( $VERSION );
$VERSION = '0.01';
# 0.01 [2007/09/28] modified string in _entry
# 0.00 [2007/08/17] separated from sb::App::Feed
# --------------------------------------
# // defines / static variables
sub OUTPUT_CHARSET (){ 'utf-8' };
sub LINK_TO_MORE   (){ '<p><a href="%s">%s</a></p>' };
sub TIME_FORMAT    (){ '%Year%-%Mon%-%Day%T%Hour%:%Min%:%Sec%' };
# --------------------------------------
# // default callbacks
sub _init
{
	return (
		'_rsspath'       => \&_main,
		'feed_entrylist' => \&_entrylist,
		'feed_entry'     => \&_entry,
	);
}
# --------------------------------------
# // define callbacks
sub _main
{
	my $cms = shift;
	my %var = @_;
	$cms->tag('feed_site_encoding'=>OUTPUT_CHARSET);
	$cms->tag('feed_url'=>sb::Content::FeedParam->get->path());
	$cms->tag('feed_date'=>
		sb::Time->format(
			'time' => $var{'time'},
			'form' => TIME_FORMAT . '+00:00',
			'zone' => '+0000',
			'lang' => 'en'
		)
	);
}
sub _entrylist
{
	my $cms = shift;
	my %var = @_;
	my $num = 0;
	foreach my $entry ( @{$var{'entry'}} )
	{
		$cms->num($num);
		$cms->tag('feed_list_url'=>$entry->permalink('cat'=>$var{'cat'}));
		$num++;
	}
	return $num;
}
sub _entry
{
	my $cms = shift;
	my %var = @_;
	my $num = 0;
	foreach my $entry ( @{$var{'entry'}} )
	{
		my $tz = $entry->tz;
		my $body = $entry->formatted_body;
		$body =~ s/&(\s|\n)/&amp;$1/g; # avoid an error of parsing xml (imperfect)
		if ($entry->more ne '')
		{
			$body .= sprintf(LINK_TO_MORE,
				$entry->permalink('cat'=>$var{'cat'},'mode'=>'more'),
				$var{'lang'}->string('[read more ...]')
			);
		}
		$cms->num($num);
		$cms->tag('feed_entry_url'=>=>$entry->permalink('cat'=>$var{'cat'}));
		$cms->tag('feed_entry_title'=>$entry->subj);
		$cms->tag('feed_entry_summary'=>sb::Text->entitize($entry->sum));
		$cms->tag('feed_entry_description'=>$body);
		if ( $entry->cat ne '' and defined($var{'cat'}->{$entry->cat}) )
		{ # category
			$cms->tag('feed_entry_category'=>$var{'cat'}->{$entry->cat}->fullname($var{'cat'}));
		}
		else
		{
			$cms->tag('feed_entry_category'=>'-');
		}
		$cms->tag('feed_entry_date'=>
			sb::Time->format(
				'time' => $entry->date,
				'form' => TIME_FORMAT . substr($tz,0,3) . ':' . substr($tz,3,2),
				'zone' => $tz,
				'lang' => 'en'
			)
		);
		$cms->tag('feed_entry_modified'=>
			sb::Time->format(
				'time' => $entry->date,
				'form' => TIME_FORMAT . 'Z',
				'zone' => '+0000',
				'lang' => 'en'
			)
		);
		$cms->tag('feed_entry_author'=>$entry->authname($var{'user'}));
		$cms->tag('product_name'=>$sb::PRODUCT);
		$cms->tag('product_webpage'=>$sb::WEBPAGE);
		$cms->tag('site_lang'=>$var{'lang'}->language);
		$num++;
	}
	return $num;
}
# ======================================
package sb::Content::FeedParam;
use base qw( sb::RequestObject );
sub initialize
{
	my $self = shift;
	$self->SUPER::initialize('path' => '');
	$self->generate_accessor('path');
	return $self;
}
1;
__END__
