# sb::InitParser
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::InitParser;
use strict;
use sb::Object ();
use base qw( sb::Object );
use vars qw( $VERSION );
$VERSION = '0.05';
# 0.05 [2007/08/04] changed initialize
# 0.04 [2007/07/17] changed load and save to change file format
# 0.03 [2007/07/16] changed return to make them consistent
# 0.02 [2007/07/13] added save and changed the module name
# 0.01 [2006/07/04] got rid of "return undef"
# 0.00 [2006/04/06] created
# --------------------------------------
# // defines / static variables
# ... nothing ...
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	$self->SUPER::initialize(
		'file' => undef,
		'rule' => undef,
		@_
	);
	$self->generate_accessor('file','rule');
	return $self;
}
# --------------------------------------
# // public functions
sub load
{
	my $self = shift;
	my $targ = shift;
	my $file = $self->file;
	return if (!$file or !-r $file);
	open(INIFILE,"<$file") or return;
	binmode(INIFILE);
	while (my $line = <INIFILE>)
	{
		$line =~ tr/\x0D\x0A//d;
		next if ($line eq '' or $line =~ /^#/);
		my ($key,$val) = split("\t",$line,2);
		$key = $self->_decode($key);
		$key = &{$self->rule}($key) if ($self->rule and ref($self->rule) eq 'CODE');
		next if ($key eq '');
		if ( $val =~ /\{(.*)\t(.*)\}/ )
		{
			$targ->{$key} = {} if (! defined($targ->{$key}));
			$targ->{$key}->{$self->_decode($1)} = $self->_decode($2) if ($1 ne '');
		}
		elsif ( $val =~ /\t/ )
		{
			$targ->{$key} = [map { $self->_decode($_) } split("\t",$val)];
		}
		else
		{
			$targ->{$key} = $self->_decode($val);
		}
	}
	close(INIFILE);
	return 1;
}
sub save
{
	my $self = shift;
	my $targ = shift;
	my $file = $self->file;
	return if (!$file);
	open(INIFILE,">$file") or return;
	binmode(INIFILE);
	foreach my $key ( keys %{$targ} )
	{
		$key = &{$self->rule}($key) if ($self->rule and ref($self->rule) eq 'CODE');
		next if ($key eq '');
		if ( ref($targ->{$key}) eq 'HASH' )
		{
			my $check = undef;
			while ( my ($hkey,$hval) = each %{$targ->{$key}} )
			{
				print INIFILE $key,"\t{",$self->_encode($hkey),"\t",$self->_encode($hval),"}\n";
				$check = 1;
			}
			print INIFILE $key,"\t{\t}\n" if (!$check);
		}
		elsif ( ref($targ->{$key}) eq 'ARRAY' )
		{
			my $line_output = join("\t",map { $self->_encode($_) } @{$targ->{$key}});
			print INIFILE $key,"\t",$line_output,"\t\n";
		}
		else
		{
			print INIFILE $key,"\t",$self->_encode($targ->{$key}),"\n";
		}
	}
	close(INIFILE);
	return 1;
}
# --------------------------------------
# // private functions
sub _decode
{
	my $self = shift;
	my $data = shift;
	$data =~ s/\\(.)/$1 eq 't' and "\t" or $1 eq 'n' and "\n" or "$1"/eg;
	return $data;
}
sub _encode
{
	my $self = shift;
	my $data = shift;
	$data =~ s/\\/\\\\/g;
	$data =~ s/\t/\\t/g;
	$data =~ s/\n/\\n/g;
	return $data;
}
1;
__END__
